package com.ningdatech.kqapi.zzsfw.controller;

import cn.hutool.core.collection.CollUtil;
import com.ningdatech.kqapi.common.exception.BizException;
import com.ningdatech.kqapi.zzsfw.entity.dto.NdKqZzsfwMattersDeduplicateDTO;
import com.ningdatech.kqapi.zzsfw.entity.dto.NdKqZzsfwMenuDTO;
import com.ningdatech.kqapi.zzsfw.entity.vo.MatterTopVO;
import com.ningdatech.kqapi.zzsfw.entity.vo.MattersVO;
import com.ningdatech.kqapi.zzsfw.entity.vo.TreeVO;
import com.ningdatech.kqapi.zzsfw.manage.MatterManage;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author ZPF
 * @since 2023-10-27
 */
@Slf4j
@Validated
@RestController
@RequestMapping("/api/v1/zzsfw/")
@RequiredArgsConstructor
public class NdKqZzsfwMenuController {
    private final MatterManage matterManage;

    @GetMapping("/matters")
    public List<TreeVO> getMatters() {
        return matterManage.getMatters();
    }

    @GetMapping("/zones")
    public List<TreeVO> getZones() {
        return matterManage.getZones();
    }

    @GetMapping("/windows")
    public List<TreeVO> getWindows(@RequestParam(required = false) String zoneName) {
        return matterManage.getWindows(zoneName);
    }

    @GetMapping("/departments")
    public List<TreeVO> getDepartments(@RequestParam(required = false) String windowName,
                                       @RequestParam(required = false) String zoneName) {
        return matterManage.getDepartments(zoneName,windowName);
    }

    @GetMapping("/matter-list")
    public List<MattersVO> matterList(@RequestParam(required = false) String windowName,
                                       @RequestParam(required = false) String zoneName,
                                   @RequestParam(required = false) String department,
                                   @RequestParam(required = false) Integer type) {
        return matterManage.matterList(zoneName,windowName,department,type);
    }

    @GetMapping("/top10")
    public List<MatterTopVO> topTen() {
        return matterManage.topTen();
    }

    @PostMapping("/save")
    public String save(@RequestBody NdKqZzsfwMenuDTO dto) {
        return matterManage.save(dto);
    }

    @GetMapping("/remove-all")
    public String removeAll() {
        return matterManage.removeAll();
    }

    @PostMapping("/save-dup")
    public String saveDup(@RequestBody NdKqZzsfwMattersDeduplicateDTO dto) {
        return matterManage.saveDup(dto);
    }

    @GetMapping("/remove-all-dup")
    public String removeAllDup() {
        return matterManage.removeAllDup();
    }


    @PostMapping("/ddl")
    public String ddl(@RequestParam("sql") String sql) throws IOException {
        return matterManage.ddl(sql);
    }

    @PostMapping("/upload-menu")
    public String uploadMenu(@RequestParam("file") MultipartFile[] file) throws IOException {
        if (Objects.isNull(file) || file.length == 0) {
            throw new BizException("文件为空!");
        }
        return matterManage.uploadMenu(file[0]);
    }
}

